<?php
/**
 * New/Edit item admin screen.
 *
 * @package    AdveitsPortfolio
 * @subpackage Admin
 * @author     Adveits <support@adveits.com>
 * @copyright  Copyright (c) 2019, Adveits
 * @link       https://store.adveits.com/
 * @license    https://www.gnu.org/licenses/gpl-2.0.html
 */

/**
 * Item edit screen functionality.
 *
 * @since  1.0.0
 * @access public
 */
final class ADVEITS_PORTFOLIO_Item_Edit {
	/**
	 * Sets up the needed actions.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	private function __construct() {
		add_action( 'load-post.php',     array( $this, 'load' ) );
		add_action( 'load-post-new.php', array( $this, 'load' ) );

		// Add the help tabs.
		add_action( 'adveits_portfolio_load_item_edit', array( $this, 'add_help_tabs' ) );
	}

	/**
	 * Runs on the page load. Checks if we're viewing the item post type and adds
	 * the appropriate actions/filters for the page.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function load() {
		$screen    = get_current_screen();
		$item_type = adveits_portfolio_get_item_post_type();

		// Bail if not on the items screen.
		if ( empty( $screen->post_type ) || $item_type !== $screen->post_type )
			return;

		// Custom action for loading the edit item screen.
		do_action( 'adveits_portfolio_load_item_edit' );

		// Add/Remove meta boxes.
		add_action( 'add_meta_boxes', array( $this, 'add_meta_boxes' ) );
	}

	/**
	 * Adds/Removes meta boxes.
	 *
	 * @since  1.1.0
	 * @access public
	 * @param  string  $post_type
	 * @return void
	 */
	public function add_meta_boxes( $post_type ) {
		//remove_meta_box( 'postexcerpt', $post_type, 'normal' );
        remove_meta_box( 'slugdiv', $post_type, 'normal' );
	}

	/**
	 * Adds custom help tabs.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function add_help_tabs() {
		$screen = get_current_screen();

		// Title and editor help tab.
		$screen->add_help_tab(
			array(
				'id'       => 'title_editor',
				'title'    => esc_html__( 'Title and Editor', 'adveits-portfolio' ),
				'callback' => array( $this, 'help_tab_title_editor' )
			)
		);

		// Item details help tab.
		$screen->add_help_tab(
			array(
				'id'       => 'item_details',
				'title'    => esc_html__( 'Gallery', 'adveits-portfolio' ),
				'callback' => array( $this, 'help_tab_gallery_details' )
			)
		);
	}

	/**
	 * Displays the title and editor help tab.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function help_tab_title_editor() { ?>
		<ul>
			<li><?php _e( "<strong>Title:</strong> Enter a title for your project. After you enter a title, you'll see the permalink below, which you can edit.", 'adveits-portfolio' ); ?></li>
			<li><?php _e( '<strong>Editor:</strong> The editor allows you to add or edit content for your project. You can insert text, media, or shortcodes.', 'adveits-portfolio' ); ?></li>
		</ul>
	<?php }

	/**
	 * Displays the item details help tab.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function help_tab_gallery_details() { ?>
		<p>
			<?php esc_html_e( 'The Gallery field group allows you to add multiple images to your project.', 'adveits-portfolio' ); ?>
		</p>

		<ul>
			<li><?php _e( '<strong>Images:</strong> Add your images using Images field.', 'adveits-portfolio' ); ?></li>
		</ul>
	<?php }

	/**
	 * Returns the instance.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return object
	 */
	public static function get_instance() {
		static $instance = null;

		if ( is_null( $instance ) )
			$instance = new self;

		return $instance;
	}
}

ADVEITS_PORTFOLIO_Item_Edit::get_instance();
