<?php
/**
 * Manage items admin screen.
 *
 * @package    AdveitsPortfolio
 * @subpackage Admin
 * @author     Adveits <support@adveits.com>
 * @copyright  Copyright (c) 2019, Adveits
 * @link       https://store.adveits.com/
 * @license    https://www.gnu.org/licenses/gpl-2.0.html
 */

/**
 * Adds additional columns and features to the items admin screen.
 *
 * @since  1.0.0
 * @access public
 */
final class ADVEITS_PORTFOLIO_Manage_Items {
	/**
	 * Sets up the needed actions.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	private function __construct() {
		add_action( 'load-edit.php', array( $this, 'load' ) );

		// Add the help tabs.
		add_action( 'adveits_portfolio_load_manage_items', array( $this, 'add_help_tabs' ) );
	}

	/**
	 * Runs on the page load. Checks if we're viewing the item post type and adds
	 * the appropriate actions/filters for the page.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function load() {
		$screen    = get_current_screen();
		$item_type = adveits_portfolio_get_item_post_type();

		// Bail if not on the items screen.
		if ( empty( $screen->post_type ) || $item_type !== $screen->post_type )
			return;

		// Custom action for loading the manage items screen.
		do_action( 'adveits_portfolio_load_manage_items' );

		// Category and tag table filters.
		add_action( 'restrict_manage_posts', array( $this, 'categories_dropdown' ) );
		add_action( 'restrict_manage_posts', array( $this, 'tags_dropdown'       ) );

		// Custom columns on the edit portfolio items screen.
		add_filter( "manage_edit-{$item_type}_columns",        array( $this, 'columns' )              );
		add_action( "manage_{$item_type}_posts_custom_column", array( $this, 'custom_column' ), 10, 2 );

		// Print custom styles.
		add_action( 'admin_head', array( $this, 'print_styles' ) );

		// Filter the row actions (shown below title).
		add_filter( 'post_row_actions', array( $this, 'row_actions' ), 10, 2 );
	}

	/**
	 * Print styles.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function print_styles() { ?>
		<style type="text/css">@media only screen and (min-width: 783px) {
			.fixed .column-thumbnail { width: 100px; }
			.fixed .column-taxonomy-<?php echo esc_attr( adveits_portfolio_get_category_taxonomy() ); ?>,
			.fixed .column-taxonomy-<?php echo esc_attr( adveits_portfolio_get_tag_taxonomy() ); ?> { width: 15%; }
		}</style>
	<?php }

	/**
	 * Renders a categories dropdown below the table nav.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function categories_dropdown() {
		$this->terms_dropdown( adveits_portfolio_get_category_taxonomy() );
	}

	/**
	 * Renders a tags dropdown below the table nav.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function tags_dropdown() {
		$this->terms_dropdown( adveits_portfolio_get_tag_taxonomy() );
	}

	/**
	 * Renders a terms dropdown based on the given taxonomy.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function terms_dropdown( $taxonomy ) {
		wp_dropdown_categories(
			array(
				'show_option_all'   => false,
				'show_option_none'  => get_taxonomy( $taxonomy )->labels->all_items,
				'option_none_value' => '',
				'orderby'           => 'name',
				'order'             => 'ASC',
				'show_count'        => true,
				'selected'          => isset( $_GET[ $taxonomy ] ) ? esc_attr( $_GET[ $taxonomy ] ) : '',
				'hierarchical'      => true,
				'name'              => $taxonomy,
				'id'                => '',
				'class'             => 'postform',
				'taxonomy'          => $taxonomy,
				'hide_if_empty'     => true,
				'value_field'	    => 'slug',
			)
		);
	}

	/**
	 * Sets up custom columns on the items edit screen.
	 *
	 * @since  1.0.0
	 * @access public
	 * @param  array  $columns
	 * @return array
	 */
	public function columns( $columns ) {
		$new_columns = array(
			'cb'    => $columns['cb'],
			'title' => __( 'Project', 'adveits-portfolio' )
		);

		if ( current_theme_supports( 'post-thumbnails' ) )
			$new_columns['thumbnail'] = __( 'Thumbnail', 'adveits-portfolio' );

		$columns = array_merge( $new_columns, $columns );

		$columns['title'] = $new_columns['title'];

		return $columns;
	}

	/**
	 * Displays the content of custom item columns on the edit screen.
	 *
	 * @since  1.0.0
	 * @access public
	 * @param  string  $column
	 * @param  int     $post_id
	 * @return void
	 */
	public function custom_column( $column, $post_id ) {
		if ( 'thumbnail' === $column ) {

			if ( has_post_thumbnail() )
				the_post_thumbnail( array( 75, 75 ) );

			elseif ( function_exists( 'get_the_image' ) )
				get_the_image( array( 'scan' => true, 'width' => 75, 'link' => false ) );
		}
	}

	/**
	 * Custom row actions below the post title.
	 *
	 * @since  1.0.0
	 * @access public
	 * @param  array   $actions
	 * @param  object  $post
	 * @return array
	 */
	function row_actions( $actions, $post ) {
		$post_type_object = get_post_type_object( adveits_portfolio_get_item_post_type() );
		$item_id = adveits_portfolio_get_item_id( $post->ID );

		if ( 'trash' === get_post_status( $item_id ) || ! current_user_can( $post_type_object->cap->publish_posts ) )
			return $actions;

		// Move view action to the end.
		if ( isset( $actions['view'] ) ) {
			$view_action = $actions['view'];
			unset( $actions['view'] );

			$actions['view'] = $view_action;
		}

		return $actions;
	}

	/**
	 * Adds custom help tabs.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function add_help_tabs() {
		$screen = get_current_screen();

		// Overview help tab.
		$screen->add_help_tab(
			array(
				'id'       => 'overview',
				'title'    => esc_html__( 'Overview', 'adveits-portfolio' ),
				'callback' => array( $this, 'help_tab_overview' )
			)
		);

		// Screen content help tab.
		$screen->add_help_tab(
			array(
				'id'       => 'screen_content',
				'title'    => esc_html__( 'Screen Content', 'adveits-portfolio' ),
				'callback' => array( $this, 'help_tab_screen_content' )
			)
		);

		// Available actions help tab.
		$screen->add_help_tab(
			array(
				'id'       => 'available_actions',
				'title'    => esc_html__( 'Available Actions', 'adveits-portfolio' ),
				'callback' => array( $this, 'help_tab_available_actions' )
			)
		);
	}

	/**
	 * Displays the overview help tab.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function help_tab_overview() { ?>
		<p>
			<?php esc_html_e( 'This screen provides access to all of your portfolio projects. You can customize the display of this screen to suit your workflow.', 'adveits-portfolio' ); ?>
		</p>
	<?php }

	/**
	 * Displays the screen content help tab.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function help_tab_screen_content() { ?>
		<p>
			<?php esc_html_e( "You can customize the display of this screen's contents in a number of ways:", 'adveits-portfolio' ); ?>
		</p>

		<ul>
			<li><?php esc_html_e( 'You can hide/display columns based on your needs and decide how many projects to list per screen using the Screen Options tab.', 'adveits-portfolio' ); ?></li>
			<li><?php esc_html_e( 'You can filter the list of projects by post status using the text links in the upper left to show All, Published, Draft, or Trashed projects. The default view is to show all projects.', 'adveits-portfolio' ); ?></li>
			<li><?php esc_html_e( 'You can view projects in a simple title list or with an excerpt. Choose the view you prefer by clicking on the icons at the top of the list on the right.', 'adveits-portfolio' ); ?></li>
			<li><?php esc_html_e( 'You can refine the list to show only projects in a specific category, with a specific tag, or from a specific month by using the dropdown menus above the projects list. Click the Filter button after making your selection. You also can refine the list by clicking on the project category or tag in the posts list.', 'adveits-portfolio' ); ?></li>
		</ul>
	<?php }

	/**
	 * Displays the available actions help tab.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function help_tab_available_actions() { ?>
		<p>
			<?php esc_html_e( 'Hovering over a row in the projects list will display action links that allow you to manage your project. You can perform the following actions:', 'adveits-portfolio' ); ?>
		</p>

		<ul>
			<li><?php _e( '<strong>Edit</strong> takes you to the editing screen for that project. You can also reach that screen by clicking on the project title.', 'adveits-portfolio' ); ?></li>
			<li><?php _e( '<strong>Quick Edit</strong> provides inline access to the metadata of your project, allowing you to update project details without leaving this screen.', 'adveits-portfolio' ); ?></li>
			<li><?php _e( '<strong>Trash</strong> removes your project from this list and places it in the trash, from which you can permanently delete it.', 'adveits-portfolio' ); ?></li>
			<li><?php _e( "<strong>Preview</strong> will show you what your draft project will look like if you publish it. View will take you to your live site to view the project. Which link is available depends on your project's status.", 'adveits-portfolio' ); ?></li>
		</ul>
	<?php }

	/**
	 * Returns the instance.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return object
	 */
	public static function get_instance() {
		static $instance = null;

		if ( is_null( $instance ) )
			$instance = new self;

		return $instance;
	}
}

ADVEITS_PORTFOLIO_Manage_Items::get_instance();
