<?php
/**
 * Plugin settings screen.
 *
 * @package    AdveitsPortfolio
 * @subpackage Admin
 * @author     Adveits <support@adveits.com>
 * @copyright  Copyright (c) 2019, Adveits
 * @link       https://store.adveits.com/
 * @license    https://www.gnu.org/licenses/gpl-2.0.html
 */

/**
 * Sets up and handles the plugin settings screen.
 *
 * @since  1.0.0
 * @access public
 */
final class ADVEITS_PORTFOLIO_Settings_Page {
	/**
	 * Settings page name.
	 *
	 * @since  1.0.0
	 * @access public
	 * @var    string
	 */
	public $settings_page = '';

	/**
	 * Sets up the needed actions for adding and saving the meta boxes.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	private function __construct() {
		add_action( 'admin_menu', array( $this, 'admin_menu' ) );
	}

	/**
	 * Sets up custom admin menus.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function admin_menu() {
		// Create the settings page.
		$this->settings_page = add_submenu_page(
			'edit.php?post_type=' . adveits_portfolio_get_item_post_type(),
			esc_html__( 'Portfolio Settings', 'adveits-portfolio' ),
			esc_html__( 'Settings', 'adveits-portfolio' ),
			apply_filters( 'adveits_portfolio_settings_capability', 'manage_options' ),
			'adveits-portfolio-settings',
			array( $this, 'settings_page' )
		);

		if ( $this->settings_page ) {
			// Register settings.
			add_action( 'admin_init', array( $this, 'register_settings' ) );

			// Add help tabs.
			add_action( "load-{$this->settings_page}", array( $this, 'add_help_tabs' ) );
		}
	}

	/**
	 * Registers the plugin settings.
	 *
	 * @since  1.1.0
	 * @access public
	 * @return void
	 */
	function register_settings() {
		// Register the setting.
		register_setting( 'adveits_portfolio_settings', 'adveits_portfolio_settings', array( $this, 'validate_settings' ) );

		/* === Settings Sections === */

		add_settings_section( 'general', esc_html__( 'General Settings', 'adveits-portfolio' ), array( $this, 'section_general' ), $this->settings_page );
		add_settings_section( 'permalinks', esc_html__( 'Permalinks', 'adveits-portfolio' ), array( $this, 'section_permalinks' ), $this->settings_page );

		/* === Settings Fields === */

		// General section fields
		add_settings_field( 'portfolio_title', esc_html__( 'Title', 'adveits-portfolio' ), array( $this, 'field_portfolio_title' ), $this->settings_page, 'general' );

		// Permalinks section fields.
		add_settings_field( 'portfolio_rewrite_base', esc_html__( 'Portfolio Base', 'adveits-portfolio' ), array( $this, 'field_portfolio_rewrite_base' ), $this->settings_page, 'permalinks' );
		add_settings_field( 'item_rewrite_base', esc_html__( 'Project Slug', 'adveits-portfolio' ), array( $this, 'field_item_rewrite_base' ), $this->settings_page, 'permalinks' );
		add_settings_field( 'category_rewrite_base', esc_html__( 'Category Slug', 'adveits-portfolio' ), array( $this, 'field_category_rewrite_base' ), $this->settings_page, 'permalinks' );
		add_settings_field( 'tag_rewrite_base', esc_html__( 'Tag Slug', 'adveits-portfolio' ), array( $this, 'field_tag_rewrite_base' ), $this->settings_page, 'permalinks' );

        // Polylang OR WPML Register Fields
        if ( function_exists( 'pll_register_string' ) ) {
            pll_register_string( 'adveits_portfolio_settings', adveits_portfolio_get_portfolio_title() );
        } else if ( class_exists( 'SitePress' ) ) {
            do_action( 'wpml_register_single_string', 'Adveits Portfolio Items', adveits_portfolio_get_portfolio_title(), adveits_portfolio_get_portfolio_title() );
        }
	}

	/**
	 * Validates the plugin settings.
	 *
	 * @since  1.0.0
	 * @access public
	 * @param  array  $input
	 * @return array
	 */
	function validate_settings( $settings ) {
		// Text boxes.
		$settings['portfolio_rewrite_base'] = $settings['portfolio_rewrite_base'] ? trim( strip_tags( $settings['portfolio_rewrite_base'] ), '/' ) : 'portfolio';
		$settings['item_rewrite_base']      = $settings['item_rewrite_base'] ? trim( strip_tags( $settings['item_rewrite_base'] ), '/' ) : '';
		$settings['category_rewrite_base']  = $settings['category_rewrite_base'] ? trim( strip_tags( $settings['category_rewrite_base'] ), '/' ) : '';
		$settings['tag_rewrite_base']       = $settings['tag_rewrite_base'] ? trim( strip_tags( $settings['tag_rewrite_base'] ), '/' ) : '';
		$settings['portfolio_title']        = $settings['portfolio_title'] ? strip_tags( $settings['portfolio_title'] ) : esc_html__( 'Portfolio', 'adveits-portfolio' );

		/* === Handle Permalink Conflicts ===*/

		// No item or category base, items win.
		if ( ! $settings['item_rewrite_base'] && ! $settings['category_rewrite_base'] )
			$settings['category_rewrite_base'] = 'categories';

		// No item or tag base, items win.
		if ( ! $settings['item_rewrite_base'] && ! $settings['tag_rewrite_base'] )
			$settings['tag_rewrite_base'] = 'tags';

		// No category or tag base, categories win.
		if ( ! $settings['category_rewrite_base'] && ! $settings['tag_rewrite_base'] )
			$settings['tag_rewrite_base'] = 'tags';

		// Return the validated/sanitized settings.
		return $settings;
	}

	/**
	 * General section callback.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function section_general() { ?>
		<p class="description">
			<?php esc_html_e( 'General portfolio settings for your site.', 'adveits-portfolio' ); ?>
		</p>
	<?php }

	/**
	 * Portfolio title field callback.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function field_portfolio_title() { ?>
		<label>
			<input type="text" class="regular-text" name="adveits_portfolio_settings[portfolio_title]" value="<?php echo esc_attr( adveits_portfolio_get_portfolio_title() ); ?>" />
			<br />
			<span class="description"><?php esc_html_e( 'The name of your portfolio. May be used for the portfolio page title and other places, depending on your theme.', 'adveits-portfolio' ); ?></span>
		</label>
	<?php }

	/**
	 * Permalinks section callback.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function section_permalinks() { ?>
		<p class="description">
			<?php esc_html_e( 'Set up custom permalinks for the portfolio section on your site.', 'adveits-portfolio' ); ?>
		</p>
	<?php }

	/**
	 * Portfolio rewrite base field callback.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function field_portfolio_rewrite_base() { ?>
		<label>
			<code><?php echo esc_url( home_url( '/' ) ); ?></code>
			<input type="text" class="regular-text code" name="adveits_portfolio_settings[portfolio_rewrite_base]" value="<?php echo esc_attr( adveits_portfolio_get_portfolio_rewrite_base() ); ?>" />
		</label>
	<?php }

	/**
	 * Portfolio rewrite base field callback.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function field_item_rewrite_base() { ?>
		<label>
			<code><?php echo esc_url( home_url( adveits_portfolio_get_portfolio_rewrite_base() . '/' ) ); ?></code>
			<input type="text" class="regular-text code" name="adveits_portfolio_settings[item_rewrite_base]" value="<?php echo esc_attr( adveits_portfolio_get_item_rewrite_base() ); ?>" />
		</label>
	<?php }

	/**
	 * Portfolio rewrite base field callback.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function field_category_rewrite_base() { ?>
		<label>
			<code><?php echo esc_url( home_url( adveits_portfolio_get_portfolio_rewrite_base() . '/' ) ); ?></code>
			<input type="text" class="regular-text code" name="adveits_portfolio_settings[category_rewrite_base]" value="<?php echo esc_attr( adveits_portfolio_get_category_rewrite_base() ); ?>" />
		</label>
	<?php }

	/**
	 * Portfolio rewrite base field callback.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function field_tag_rewrite_base() { ?>
		<label>
			<code><?php echo esc_url( home_url( adveits_portfolio_get_portfolio_rewrite_base() . '/' ) ); ?></code>
			<input type="text" class="regular-text code" name="adveits_portfolio_settings[tag_rewrite_base]" value="<?php echo esc_attr( adveits_portfolio_get_tag_rewrite_base() ); ?>" />
		</label>
	<?php }

	/**
	 * Renders the settings page.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function settings_page() {
		// Flush the rewrite rules if the settings were updated.
		if ( isset( $_GET['settings-updated'] ) )
			flush_rewrite_rules(); ?>

		<div class="wrap">
			<h1><?php esc_html_e( 'Portfolio Settings', 'adveits-portfolio' ); ?></h1>

			<?php settings_errors(); ?>

			<form method="post" action="options.php">
				<?php settings_fields( 'adveits_portfolio_settings' ); ?>
				<?php do_settings_sections( $this->settings_page ); ?>
				<?php submit_button( esc_attr__( 'Update Settings', 'adveits-portfolio' ), 'primary' ); ?>
			</form>

		</div><!-- wrap -->
	<?php }

	/**
	 * Adds help tabs.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function add_help_tabs() {
		// Get the current screen.
		$screen = get_current_screen();

		// General settings help tab.
		$screen->add_help_tab(
			array(
				'id'       => 'general',
				'title'    => esc_html__( 'General Settings', 'adveits-portfolio' ),
				'callback' => array( $this, 'help_tab_general' )
			)
		);

		// Permalinks settings help tab.
		$screen->add_help_tab(
			array(
				'id'       => 'permalinks',
				'title'    => esc_html__( 'Permalinks', 'adveits-portfolio' ),
				'callback' => array( $this, 'help_tab_permalinks' )
			)
		);
	}

	/**
	 * Displays the general settings help tab.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function help_tab_general() { ?>
		<ul>
			<li><?php _e( '<strong>Title:</strong> Allows you to set the title for the portfolio section on your site. This is general shown on the portfolio projects archive, but themes and other plugins may use it in other ways.', 'adveits-portfolio' ); ?></li>
		</ul>
	<?php }

	/**
	 * Displays the permalinks help tab.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function help_tab_permalinks() { ?>
		<ul>
			<li><?php _e( '<strong>Portfolio Base:</strong> The primary URL for the portfolio section on your site. It lists your portfolio projects.', 'adveits-portfolio' ); ?></li>
			<li>
				<?php _e( '<strong>Project Slug:</strong> The slug for single portfolio projects. You can use something custom, leave this field empty, or use one of the following tags:', 'adveits-portfolio' ); ?>
				<ul>
					<li><?php printf( esc_html__( '%s - The project category.', 'adveits-portfolio' ), '<code>%' . adveits_portfolio_get_category_taxonomy() . '%</code>' ); ?></li>
					<li><?php printf( esc_html__( '%s - The project tag.', 'adveits-portfolio' ), '<code>%' . adveits_portfolio_get_tag_taxonomy() . '%</code>' ); ?></li>
				</ul>
			</li>
			<li><?php _e( '<strong>Category Slug:</strong> The base slug used for portfolio category archives.', 'adveits-portfolio' ); ?></li>
			<li><?php _e( '<strong>Tag Slug:</strong> The base slug used for portfolio tag archives.', 'adveits-portfolio' ); ?></li>
		</ul>
	<?php }

	/**
	 * Returns the instance.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return object
	 */
	public static function get_instance() {
		static $instance = null;

		if ( is_null( $instance ) )
			$instance = new self;

		return $instance;
	}
}

ADVEITS_PORTFOLIO_Settings_Page::get_instance();
