<?php
/**
 * Admin-related functions and filters.
 *
 * @package    AdveitsPortfolio
 * @subpackage Admin
 * @author     Adveits <support@adveits.com>
 * @copyright  Copyright (c) 2019, Adveits
 * @link       https://store.adveits.com/
 * @license    https://www.gnu.org/licenses/gpl-2.0.html
 */

# Filter post format support for items.
add_action( 'load-post.php',     'adveits_portfolio_post_format_support_filter' );
add_action( 'load-post-new.php', 'adveits_portfolio_post_format_support_filter' );
add_action( 'load-edit.php',     'adveits_portfolio_post_format_support_filter' );

/**
 * Helper function for getting the correct slug for the settings page.  This is useful
 * for add-on plugins that need to add custom setting sections or fields to the settings
 * screen for the plugin.
 *
 * @since  1.0.0
 * @access public
 * @return string
 */
function adveits_portfolio_get_settings_page_slug() {
	return sprintf( '%s_page_adveits-portfolio-settings', adveits_portfolio_get_item_post_type() );
}

/**
 * Returns an array of post formats allowed for the item post type.
 *
 * @since  1.0.0
 * @access public
 * @return array
 */
function adveits_portfolio_get_allowed_item_formats() {
	return apply_filters( 'adveits_portfolio_get_allowed_item_formats', array( 'audio', 'gallery', 'image', 'video' ) );
}

/**
 * If a theme supports post formats, limit item to only only the audio, image,
 * gallery, and video formats.
 *
 * @since  1.0.0
 * @access public
 * @return void
 */
function adveits_portfolio_post_format_support_filter() {
	$screen    = get_current_screen();
	$item_type = adveits_portfolio_get_item_post_type();

	// Bail if not on the items screen.
	if ( empty( $screen->post_type ) || $item_type !== $screen->post_type )
		return;

	// Check if the current theme supports formats.
	if ( current_theme_supports( 'post-formats' ) ) {

		$formats = get_theme_support( 'post-formats' );

		// If we have formats, add theme support for only the allowed formats.
		if ( isset( $formats[0] ) ) {
			$new_formats = array_intersect( $formats[0], adveits_portfolio_get_allowed_item_formats() );

			// Remove post formats support.
			remove_theme_support( 'post-formats' );

			// If the theme supports the allowed formats, add support for them.
			if ( $new_formats )
				add_theme_support( 'post-formats', $new_formats );
		}
	}

	// Filter the default post format.
	add_filter( 'option_default_post_format', 'adveits_portfolio_default_post_format_filter', 95 );
}

/**
 * Filters the default post format to make sure that it's in our list of supported formats.
 *
 * @since  1.0.0
 * @access public
 * @param  string  $format
 * @return string
 */
function adveits_portfolio_default_post_format_filter( $format ) {
	return in_array( $format, adveits_portfolio_get_allowed_item_formats() ) ? $format : 'standard';
}
