<?php
/**
 * Various functions, filters, and actions used by the plugin.
 *
 * @package    AdveitsPortfolio
 * @subpackage Includes
 * @author     Adveits <support@adveits.com>
 * @copyright  Copyright (c) 2019, Adveits
 * @link       https://store.adveits.com/
 * @license    https://www.gnu.org/licenses/gpl-2.0.html
 */

# Template hierarchy.
add_filter( 'template_include', 'adveits_portfolio_template_include', 5 );

# Filter the post type archive title.
add_filter( 'post_type_archive_title', 'adveits_portfolio_post_type_archive_title', 5, 2 );

# Filter the archive title and description.
add_filter( 'get_the_archive_title', 'adveits_portfolio_get_the_archive_title', 5 );

# Filter the post type permalink.
add_filter( 'post_type_link', 'adveits_portfolio_post_type_link', 10, 2 );

# Force taxonomy term selection.
add_action( 'save_post', 'adveits_portfolio_force_term_selection' );

# Filter the Breadcrumb Trail plugin args.
add_filter( 'breadcrumb_trail_args', 'adveits_portfolio_breadcrumb_trail_args', 15 );

/**
 * Basic top-level template hierarchy. I generally prefer to leave this functionality up to
 * themes.  This is just a foundation to build upon if needed.
 *
 * @since  1.1.0
 * @access public
 * @param string $template
 * @return string
 */
function adveits_portfolio_template_include( $template ) {
    // Bail if not a portfolio page.
    if ( ! adveits_portfolio_is_portfolio() )
        return $template;
    
    $templates_new = array();
    
    if ( adveits_portfolio_is_category() ) {
        $templates_new[] = 'template-plugins/adveits-portfolio/portfolio-category.php';
        $templates_new[] = 'template-plugins/adveits-portfolio/portfolio-archive.php';
        // Tag archive.
    } else if ( adveits_portfolio_is_tag() ) {
        $templates_new[] = 'template-plugins/adveits-portfolio/portfolio-tag.php';
        $templates_new[] = 'template-plugins/adveits-portfolio/portfolio-archive.php';
        // portfolio archive.
    } else if ( adveits_portfolio_is_item_archive() ) {
        $templates_new[] = 'template-plugins/adveits-portfolio/portfolio-archive.php';
        // Single item.
    } else if ( adveits_portfolio_is_single_item() ) {
        $templates_new[] = 'template-plugins/adveits-portfolio/portfolio-single.php';
    }
    
    $templates_old = array();
    
    if ( adveits_portfolio_is_category() ) {
        $templates_old[] = 'portfolio-category.php';
        $templates_old[] = 'portfolio-archive.php';
        // Tag archive.
    } else if ( adveits_portfolio_is_tag() ) {
        $templates_old[] = 'portfolio-tag.php';
        $templates_old[] = 'portfolio-archive.php';
        // portfolio archive.
    } else if ( adveits_portfolio_is_item_archive() ) {
        $templates_old[] = 'portfolio-archive.php';
        // Single item.
    } else if ( adveits_portfolio_is_single_item() ) {
        $templates_old[] = 'portfolio-single.php';
    }
    
    // Fallback template.
    $templates_new[] = 'portfolio.php';
    $templates_old[] = 'portfolio.php';
    
    // Check if we have a template.
    $has_template_new = locate_template( apply_filters( 'adveits_portfolio_template_hierarchy', $templates_new ) );
    $has_template_old = locate_template( apply_filters( 'adveits_portfolio_template_hierarchy', $templates_old ) );
    
    // Return the template.
    if ( $has_template_new ) {
        return $has_template_new;
    } else if ( $has_template_old ) {
        return $has_template_old;
    } else {
        return $template;
    }
}

/**
 * Filter on 'post_type_archive_title' to allow for the use of the 'archive_title' label that isn't supported
 * by WordPress.  That's okay since we can roll our own labels.
 *
 * @since  1.0.0
 * @access public
 * @param string $title
 * @param string $post_type
 * @return string
 */
function adveits_portfolio_post_type_archive_title( $title, $post_type ) {
    $item_type = adveits_portfolio_get_item_post_type();
    
    return $item_type === $post_type ? get_post_type_object( adveits_portfolio_get_item_post_type() )->labels->archive_title : $title;
}

/**
 * Filters the archive title. Note that we need this additional filter because core WP does
 * things like add "Archives:" in front of the archive title.
 *
 * @since  1.0.0
 * @access public
 * @param string $title
 * @return string
 */
function adveits_portfolio_get_the_archive_title( $title ) {
    if ( adveits_portfolio_is_item_archive() )
        $title = post_type_archive_title( '', false );
    
    return $title;
}

/**
 * Filter on `post_type_link` to make sure that single portfolio items have the correct
 * permalink.
 *
 * @since  1.0.0
 * @access public
 * @param string $post_link
 * @param object $post
 * @return string
 */
function adveits_portfolio_post_type_link( $post_link, $post ) {
    // Bail if this isn't a portfolio item.
    if ( adveits_portfolio_get_item_post_type() !== $post->post_type )
        return $post_link;
    
    $cat_taxonomy = adveits_portfolio_get_category_taxonomy();
    $tag_taxonomy = adveits_portfolio_get_tag_taxonomy();
    
    $category = $tag = '';
    
    // Check for the category.
    if ( false !== strpos( $post_link, "%{$cat_taxonomy}%" ) ) {
        // Get the terms.
        $terms = get_the_terms( $post, $cat_taxonomy );
        
        // Check that terms were returned.
        if ( $terms ) {
            if ( function_exists( 'wp_list_sort' ) ) {
                $terms = wp_list_sort( $terms, 'term_id', 'ASC' );
            } else {
                usort( $terms, '_usort_terms_by_ID' );
            }
            
            $category = $terms[0]->slug;
        }
    }
    
    // Check for the tag.
    if ( false !== strpos( $post_link, "%{$tag_taxonomy}%" ) ) {
        // Get the terms.
        $terms = get_the_terms( $post, $tag_taxonomy );
        
        // Check that terms were returned.
        if ( $terms ) {
            if ( function_exists( 'wp_list_sort' ) ) {
                $terms = wp_list_sort( $terms, 'term_id', 'ASC' );
            } else {
                usort( $terms, '_usort_terms_by_ID' );
            }
            
            $tag = $terms[0]->slug;
        }
    }
    
    $rewrite_tags = array(
        '%adveits_portfolio_category%',
        '%adveits_portfolio_tag%'
    );
    
    $map_tags = array(
        $category,
        $tag
    );
    
    return str_replace( $rewrite_tags, $map_tags, $post_link );
}

/**
 * If a item has `%adveits_portfolio_category%` or `%adveits_portfolio_tag%` in its permalink structure,
 * it must have a term set for the taxonomy. This function is a callback on `save_post`
 * that checks if a term is set. If not, it forces the first term of the taxonomy to be
 * the selected term.
 *
 * @since  1.0.0
 * @access public
 * @param int $post_id
 * @return void
 */
function adveits_portfolio_force_term_selection( $post_id ) {
    if ( adveits_portfolio_is_item( $post_id ) ) {
        $item_base = adveits_portfolio_get_item_rewrite_base();
        $cat_tax   = adveits_portfolio_get_category_taxonomy();
        $tag_tax   = adveits_portfolio_get_tag_taxonomy();
        
        if ( false !== strpos( $item_base, "%{$cat_tax}%" ) )
            adveits_portfolio_set_term_if_none( $post_id, $cat_tax, adveits_portfolio_get_default_category() );
        
        if ( false !== strpos( $item_base, "%{$tag_tax}%" ) )
            adveits_portfolio_set_term_if_none( $post_id, $tag_tax, adveits_portfolio_get_default_tag() );
    }
}

/**
 * Checks if a post has a term of the given taxonomy. If not, set it with the first
 * term available from the taxonomy.
 *
 * @since  1.0.0
 * @access public
 * @param int $post_id
 * @param string $taxonomy
 * @param int $default
 * @return void
 */
function adveits_portfolio_set_term_if_none( $post_id, $taxonomy, $default = 0 ) {
    // Get the current post terms.
    $terms = wp_get_post_terms( $post_id, $taxonomy );
    
    // If no terms are set, let's roll.
    if ( ! $terms ) {
        $new_term = false;
        
        // Get the default term if set.
        if ( $default )
            $new_term = get_term( $default, $taxonomy );
        
        // If no default term or if there's an error, get the first term.
        if ( ! $new_term || is_wp_error( $new_term ) ) {
            $available = get_terms( $taxonomy, array( 'number' => 1 ) );
            
            // Get the first term.
            $new_term = $available ? array_shift( $available ) : false;
        }
        
        // Only run if there are taxonomy terms.
        if ( $new_term ) {
            $tax_object = get_taxonomy( $taxonomy );
            
            // Use the ID for hierarchical taxonomies. Use the slug for non-hierarchical.
            $slug_or_id = $tax_object->hierarchical ? $new_term->term_id : $new_term->slug;
            
            // Set the new post term.
            wp_set_post_terms( $post_id, $slug_or_id, $taxonomy, true );
        }
    }
}

/**
 * Filters the Breadcrumb Trail plugin arguments. We're basically just telling it to show the
 * `portfolio_category` taxonomy when viewing single portfolio items.
 *
 * @since  1.0.0
 * @access public
 * @param array $args
 * @return array
 */
function adveits_portfolio_breadcrumb_trail_args( $args ) {
    $item_type = adveits_portfolio_get_item_post_type();
    $item_base = adveits_portfolio_get_item_rewrite_base();
    
    if ( false === strpos( $item_base, '%' ) && ! isset( $args['post_taxonomy'][$item_type] ) )
        $args['post_taxonomy'][$item_type] = adveits_portfolio_get_category_taxonomy();
    
    return $args;
}
