<?php
/**
 * File for registering portfolio post types.
 *
 * @package    AdveitsPortfolio
 * @subpackage Includes
 * @author     Adveits <support@adveits.com>
 * @copyright  Copyright (c) 2019, Adveits
 * @link       https://store.adveits.com/
 * @license    https://www.gnu.org/licenses/gpl-2.0.html
 */

# Register portfolio post types on the 'init' hook.
add_action( 'init', 'adveits_portfolio_register_post_types' );

# Filter the "enter title here" text.
add_filter( 'enter_title_here', 'adveits_portfolio_enter_title_here', 10, 2 );

# Filter the bulk and post updated messages.
add_filter( 'bulk_post_updated_messages', 'adveits_portfolio_bulk_post_updated_messages', 5, 2 );
add_filter( 'post_updated_messages', 'adveits_portfolio_post_updated_messages', 5 );

/**
 * Returns the name of the item post type.
 *
 * @since  1.0.0
 * @access public
 * @return string
 */
function adveits_portfolio_get_item_post_type() {
	return apply_filters( 'adveits_portfolio_get_item_post_type', 'adveits_portfolio' );
}

/**
 * Returns the capabilities for the item post type.
 *
 * @since  1.0.0
 * @access public
 * @return array
 */
function adveits_portfolio_get_item_capabilities() {
	$caps = array(
		// meta caps (don't assign these to roles)
		'edit_post'              => 'edit_adveits_portfolio_item',
		'read_post'              => 'read_adveits_portfolio_item',
		'delete_post'            => 'delete_adveits_portfolio_item',

		// primitive/meta caps
		'create_posts'           => 'create_adveits_portfolio_items',

		// primitive caps used outside of map_meta_cap()
		'edit_posts'             => 'edit_adveits_portfolio_items',
		'edit_others_posts'      => 'edit_others_adveits_portfolio_items',
		'publish_posts'          => 'publish_adveits_portfolio_items',
		'read_private_posts'     => 'read_private_adveits_portfolio_items',

		// primitive caps used inside of map_meta_cap()
		'read'                   => 'read',
		'delete_posts'           => 'delete_adveits_portfolio_items',
		'delete_private_posts'   => 'delete_private_adveits_portfolio_items',
		'delete_published_posts' => 'delete_published_adveits_portfolio_items',
		'delete_others_posts'    => 'delete_others_adveits_portfolio_items',
		'edit_private_posts'     => 'edit_private_adveits_portfolio_items',
		'edit_published_posts'   => 'edit_published_adveits_portfolio_items'
	);

	return apply_filters( 'adveits_portfolio_get_item_capabilities', $caps );
}

/**
 * Returns the labels for the item post type.
 *
 * @since  1.0.0
 * @access public
 * @return array
 */
function adveits_portfolio_get_item_labels() {
    $archive_title = adveits_portfolio_get_portfolio_title();
    
    if ( function_exists( 'pll__' ) ) {
        $archive_title = pll__( adveits_portfolio_get_portfolio_title() );
    } else if ( class_exists( 'SitePress' ) ) {
        $archive_title = apply_filters( 'wpml_translate_single_string', adveits_portfolio_get_portfolio_title(), 'Adveits Portfolio Items', adveits_portfolio_get_portfolio_title() );
    }
    
	$labels = array(
		'name'                  => __( 'Projects', 'adveits-portfolio' ),
		'singular_name'         => __( 'Project', 'adveits-portfolio' ),
		'menu_name'             => __( 'Portfolio', 'adveits-portfolio' ),
		'name_admin_bar'        => __( 'Project', 'adveits-portfolio' ),
		'add_new'               => __( 'New Project', 'adveits-portfolio' ),
		'add_new_item'          => __( 'Add New Project', 'adveits-portfolio' ),
		'edit_item'             => __( 'Edit Project', 'adveits-portfolio' ),
		'new_item'              => __( 'New Project', 'adveits-portfolio' ),
		'view_item'             => __( 'View Project', 'adveits-portfolio' ),
		'view_items'            => __( 'View Projects', 'adveits-portfolio' ),
		'search_items'          => __( 'Search Projects', 'adveits-portfolio' ),
		'not_found'             => __( 'No projects found', 'adveits-portfolio' ),
		'not_found_in_trash'    => __( 'No projects found in trash', 'adveits-portfolio' ),
		'all_items'             => __( 'Projects', 'adveits-portfolio' ),
		'featured_image'        => __( 'Featured Image', 'adveits-portfolio' ),
		'set_featured_image'    => __( 'Set featured image', 'adveits-portfolio' ),
		'remove_featured_image' => __( 'Remove featured image', 'adveits-portfolio' ),
		'use_featured_image'    => __( 'Use as featured image', 'adveits-portfolio' ),
		'insert_into_item'      => __( 'Insert into project', 'adveits-portfolio' ),
		'uploaded_to_this_item' => __( 'Uploaded to this project', 'adveits-portfolio' ),
		'filter_items_list'     => __( 'Filter projects list', 'adveits-portfolio' ),
		'items_list_navigation' => __( 'Projects list navigation', 'adveits-portfolio' ),
		'items_list'            => __( 'Projects list', 'adveits-portfolio' ),

		// Custom labels b/c WordPress doesn't have anything to handle this.
		'archive_title'         => $archive_title,
	);

	return apply_filters( 'adveits_portfolio_get_item_labels', $labels );
}

/**
 * Registers post types needed by the plugin.
 *
 * @since  1.1.0
 * @access public
 * @return void
 */
function adveits_portfolio_register_post_types() {
	// Set up the arguments for the portfolio item post type.
	$item_args = array(
		'public'              => true,
		'publicly_queryable'  => true,
		'show_in_nav_menus'   => true,
		'show_in_admin_bar'   => true,
		'exclude_from_search' => false,
		'show_ui'             => true,
		'show_in_menu'        => true,
		'menu_position'       => 4,
		'menu_icon'           => adveits_portfolio_plugin()->img_uri . ( 'wp-adveits.svg' ),
		'can_export'          => true,
		'delete_with_user'    => false,
		'hierarchical'        => false,
		'has_archive'         => adveits_portfolio_get_portfolio_rewrite_base(),
		'query_var'           => adveits_portfolio_get_item_post_type(),
		'capability_type'     => 'adveits_portfolio',
		'map_meta_cap'        => true,
		'capabilities'        => adveits_portfolio_get_item_capabilities(),
		'labels'              => adveits_portfolio_get_item_labels(),

		// The rewrite handles the URL structure.
		'rewrite' => array(
			'slug'       => adveits_portfolio_get_item_rewrite_slug(),
			'with_front' => false,
			'pages'      => true,
			'feeds'      => true,
			'ep_mask'    => EP_PERMALINK,
		),

		// What features the post type supports.
		'supports' => array(
			'title',
			'editor',
			'excerpt',
			'thumbnail',
			'post-formats',

			// Theme/Plugin feature support.
			'custom-background', // Custom Background Extended
			'custom-header',     // Custom Header Extended
		)
	);

	// Register the post types.
	register_post_type( adveits_portfolio_get_item_post_type(), apply_filters( 'adveits_portfolio_item_post_type_args', $item_args ) );
    flush_rewrite_rules();
}

/**
 * Custom "enter title here" text.
 *
 * @since  1.0.0
 * @access public
 * @param  string  $title
 * @param  object  $post
 * @return string
 */
function adveits_portfolio_enter_title_here( $title, $post ) {
	return adveits_portfolio_get_item_post_type() === $post->post_type ? esc_html__( 'Enter project title', 'adveits-portfolio' ) : $title;
}

/**
 * Adds custom post updated messages on the edit post screen.
 *
 * @since  1.0.0
 * @access public
 * @param  array  $messages
 * @global object $post
 * @global int    $post_ID
 * @return array
 */
function adveits_portfolio_post_updated_messages( $messages ) {
	global $post, $post_ID;

	$item_type = adveits_portfolio_get_item_post_type();

	if ( $item_type !== $post->post_type )
		return $messages;

	// Get permalink and preview URLs.
	$permalink   = get_permalink( $post_ID );
	$preview_url = get_preview_post_link( $post );

	// Translators: Scheduled item date format. See http://php.net/date
	$scheduled_date = date_i18n( __( 'M j, Y @ H:i', 'adveits-portfolio' ), strtotime( $post->post_date ) );

	// Set up view links.
	$preview_link   = sprintf( ' <a target="_blank" href="%1$s">%2$s</a>', esc_url( $preview_url ), esc_html__( 'Preview project', 'adveits-portfolio' ) );
	$scheduled_link = sprintf( ' <a target="_blank" href="%1$s">%2$s</a>', esc_url( $permalink ), esc_html__( 'Preview project', 'adveits-portfolio' ) );
	$view_link      = sprintf( ' <a href="%1$s">%2$s</a>', esc_url( $permalink ), esc_html__( 'View project', 'adveits-portfolio' ) );

	// Post updated messages.
	$messages[ $item_type ] = array(
		 1 => esc_html__( 'Project updated.', 'adveits-portfolio' ) . $view_link,
		 4 => esc_html__( 'Project updated.', 'adveits-portfolio' ),
		 // Translators: %s is the date and time of the revision.
		 5 => isset( $_GET['revision'] ) ? sprintf( esc_html__( 'Project restored to revision from %s.', 'adveits-portfolio' ), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
		 6 => esc_html__( 'Project published.', 'adveits-portfolio' ) . $view_link,
		 7 => esc_html__( 'Project saved.', 'adveits-portfolio' ),
		 8 => esc_html__( 'Project submitted.', 'adveits-portfolio' ) . $preview_link,
		 9 => sprintf( esc_html__( 'Project scheduled for: %s.', 'adveits-portfolio' ), "<strong>{$scheduled_date}</strong>" ) . $scheduled_link,
		10 => esc_html__( 'Project draft updated.', 'adveits-portfolio' ) . $preview_link,
	);

	return $messages;
}

/**
 * Adds custom bulk post updated messages on the manage items screen.
 *
 * @since  1.0.0
 * @access public
 * @param  array  $messages
 * @param  array  $counts
 * @return array
 */
function adveits_portfolio_bulk_post_updated_messages( $messages, $counts ) {
	$type = adveits_portfolio_get_item_post_type();

	$messages[ $type ]['updated']   = _n( '%s project updated.', '%s projects updated.', $counts['updated'], 'adveits-portfolio' );
	$messages[ $type ]['locked']    = _n( '%s project not updated, somebody is editing it.', '%s projects not updated, somebody is editing them.', $counts['locked'], 'adveits-portfolio' );
	$messages[ $type ]['deleted']   = _n( '%s project permanently deleted.', '%s projects permanently deleted.', $counts['deleted'], 'adveits-portfolio' );
	$messages[ $type ]['trashed']   = _n( '%s project moved to the Trash.', '%s projects moved to the trash.', $counts['trashed'], 'adveits-portfolio' );
	$messages[ $type ]['untrashed'] = _n( '%s project restored from the Trash.', '%s projects restored from the trash.', $counts['untrashed'], 'adveits-portfolio' );

	return $messages;
}
