<?php
/**
 * File for registering custom taxonomies.
 *
 * @package    AdveitsPortfolio
 * @subpackage Includes
 * @author     Adveits <support@adveits.com>
 * @copyright  Copyright (c) 2019, Adveits
 * @link       https://store.adveits.com/
 * @license    https://www.gnu.org/licenses/gpl-2.0.html
 */

# Register taxonomies on the 'init' hook.
add_action( 'init', 'adveits_portfolio_register_taxonomies', 9 );

# Filter the term updated messages.
add_filter( 'term_updated_messages', 'adveits_portfolio_term_updated_messages', 5 );

/**
 * Returns the name of the portfolio category taxonomy.
 *
 * @since  1.0.0
 * @access public
 * @return string
 */
function adveits_portfolio_get_category_taxonomy() {
	return apply_filters( 'adveits_portfolio_get_category_taxonomy', 'adveits_portfolio_category' );
}

/**
 * Returns the name of the portfolio tag taxonomy.
 *
 * @since  1.0.0
 * @access public
 * @return string
 */
function adveits_portfolio_get_tag_taxonomy() {
	return apply_filters( 'adveits_portfolio_get_tag_taxonomy', 'adveits_portfolio_tag' );
}

/**
 * Returns the capabilities for the portfolio category taxonomy.
 *
 * @since  1.0.0
 * @access public
 * @return array
 */
function adveits_portfolio_get_category_capabilities() {
	$caps = array(
		'manage_terms' => 'manage_adveits_portfolio_categories',
		'edit_terms'   => 'edit_adveits_portfolio_categories',
		'delete_terms' => 'delete_adveits_portfolio_categories',
		'assign_terms' => 'assign_adveits_portfolio_categories'
	);

	return apply_filters( 'adveits_portfolio_get_category_capabilities', $caps );
}

/**
 * Returns the capabilities for the portfolio tag taxonomy.
 *
 * @since  1.0.0
 * @access public
 * @return array
 */
function adveits_portfolio_get_tag_capabilities() {
	$caps = array(
		'manage_terms' => 'manage_adveits_portfolio_tags',
		'edit_terms'   => 'edit_adveits_portfolio_tags',
		'delete_terms' => 'delete_adveits_portfolio_tags',
		'assign_terms' => 'assign_adveits_portfolio_tags',
	);

	return apply_filters( 'adveits_portfolio_get_tag_capabilities', $caps );
}

/**
 * Returns the labels for the portfolio category taxonomy.
 *
 * @since  1.0.0
 * @access public
 * @return array
 */
function adveits_portfolio_get_category_labels() {
	$labels = array(
		'name'                       => __( 'Categories', 'adveits-portfolio' ),
		'singular_name'              => __( 'Category', 'adveits-portfolio' ),
		'menu_name'                  => __( 'Categories', 'adveits-portfolio' ),
		'name_admin_bar'             => __( 'Category', 'adveits-portfolio' ),
		'search_items'               => __( 'Search Categories', 'adveits-portfolio' ),
		'popular_items'              => __( 'Popular Categories', 'adveits-portfolio' ),
		'all_items'                  => __( 'All Categories', 'adveits-portfolio' ),
		'edit_item'                  => __( 'Edit Category', 'adveits-portfolio' ),
		'view_item'                  => __( 'View Category', 'adveits-portfolio' ),
		'update_item'                => __( 'Update Category', 'adveits-portfolio' ),
		'add_new_item'               => __( 'Add New Category', 'adveits-portfolio' ),
		'new_item_name'              => __( 'New Category Name', 'adveits-portfolio' ),
		'not_found'                  => __( 'No categories found.', 'adveits-portfolio' ),
		'no_terms'                   => __( 'No categories', 'adveits-portfolio' ),
		'items_list_navigation'      => __( 'Categories list navigation', 'adveits-portfolio' ),
		'items_list'                 => __( 'Categories list', 'adveits-portfolio' ),

		// Hierarchical only.
		'select_name'                => __( 'Select Category', 'adveits-portfolio' ),
		'parent_item'                => __( 'Parent Category', 'adveits-portfolio' ),
		'parent_item_colon'          => __( 'Parent Category:', 'adveits-portfolio' ),
	);

	return apply_filters( 'adveits_portfolio_get_category_labels', $labels );
}

/**
 * Returns the labels for the portfolio tag taxonomy.
 *
 * @since  1.0.0
 * @access public
 * @return array
 */
function adveits_portfolio_get_tag_labels() {
	$labels = array(
		'name'                       => __( 'Tags', 'adveits-portfolio' ),
		'singular_name'              => __( 'Tag', 'adveits-portfolio' ),
		'menu_name'                  => __( 'Tags', 'adveits-portfolio' ),
		'name_admin_bar'             => __( 'Tag', 'adveits-portfolio' ),
		'search_items'               => __( 'Search Tags', 'adveits-portfolio' ),
		'popular_items'              => __( 'Popular Tags', 'adveits-portfolio' ),
		'all_items'                  => __( 'All Tags', 'adveits-portfolio' ),
		'edit_item'                  => __( 'Edit Tag', 'adveits-portfolio' ),
		'view_item'                  => __( 'View Tag', 'adveits-portfolio' ),
		'update_item'                => __( 'Update Tag', 'adveits-portfolio' ),
		'add_new_item'               => __( 'Add New Tag', 'adveits-portfolio' ),
		'new_item_name'              => __( 'New Tag Name', 'adveits-portfolio' ),
		'not_found'                  => __( 'No tags found.', 'adveits-portfolio' ),
		'no_terms'                   => __( 'No tags', 'adveits-portfolio' ),
		'items_list_navigation'      => __( 'Tags list navigation', 'adveits-portfolio' ),
		'items_list'                 => __( 'Tags list', 'adveits-portfolio' ),

		// Non-hierarchical only.
		'separate_items_with_commas' => __( 'Separate tags with commas', 'adveits-portfolio' ),
		'add_or_remove_items'        => __( 'Add or remove tags', 'adveits-portfolio' ),
		'choose_from_most_used'      => __( 'Choose from the most used tags', 'adveits-portfolio' ),
	);

	return apply_filters( 'adveits_portfolio_get_tag_labels', $labels );
}

/**
 * Register taxonomies for the plugin.
 *
 * @since  1.0.0
 * @access public
 * @return void.
 */
function adveits_portfolio_register_taxonomies() {
	// Set up the arguments for the portfolio category taxonomy.
	$cat_args = array(
		'public'            => true,
		'show_ui'           => true,
		'show_in_nav_menus' => true,
		'show_tagcloud'     => true,
		'show_admin_column' => true,
		'hierarchical'      => true,
		'query_var'         => adveits_portfolio_get_category_taxonomy(),
		'capabilities'      => adveits_portfolio_get_category_capabilities(),
		'labels'            => adveits_portfolio_get_category_labels(),

		// The rewrite handles the URL structure.
		'rewrite' => array(
			'slug'         => adveits_portfolio_get_category_rewrite_slug(),
			'with_front'   => false,
			'hierarchical' => false,
			'ep_mask'      => EP_NONE
		),
	);

	// Set up the arguments for the portfolio tag taxonomy.
	$tag_args = array(
		'public'            => true,
		'show_ui'           => true,
		'show_in_nav_menus' => true,
		'show_tagcloud'     => true,
		'show_admin_column' => true,
		'hierarchical'      => false,
		'query_var'         => adveits_portfolio_get_tag_taxonomy(),
		'capabilities'      => adveits_portfolio_get_tag_capabilities(),
		'labels'            => adveits_portfolio_get_tag_labels(),

		// The rewrite handles the URL structure.
		'rewrite' => array(
			'slug'         => adveits_portfolio_get_tag_rewrite_slug(),
			'with_front'   => false,
			'hierarchical' => false,
			'ep_mask'      => EP_NONE
		),
	);

	// Register the taxonomies.
	register_taxonomy( adveits_portfolio_get_category_taxonomy(), adveits_portfolio_get_item_post_type(), apply_filters( 'adveits_portfolio_category_taxonomy_args', $cat_args ) );
	register_taxonomy( adveits_portfolio_get_tag_taxonomy(), adveits_portfolio_get_item_post_type(), apply_filters( 'adveits_portfolio_tag_taxonomy_args', $tag_args ) );
}

/**
 * Filters the term updated messages in the admin.
 *
 * @since  1.0.0
 * @access public
 * @param  array  $messages
 * @return array
 */
function adveits_portfolio_term_updated_messages( $messages ) {
	$cat_taxonomy = adveits_portfolio_get_category_taxonomy();
	$tag_taxonomy = adveits_portfolio_get_tag_taxonomy();

	// Add the portfolio category messages.
	$messages[ $cat_taxonomy ] = array(
		0 => '',
		1 => __( 'Category added.', 'adveits-portfolio' ),
		2 => __( 'Category deleted.', 'adveits-portfolio' ),
		3 => __( 'Category updated.', 'adveits-portfolio' ),
		4 => __( 'Category not added.', 'adveits-portfolio' ),
		5 => __( 'Category not updated.', 'adveits-portfolio' ),
		6 => __( 'Categories deleted.', 'adveits-portfolio' ),
	);

	// Add the portfolio tag messages.
	$messages[ $tag_taxonomy ] = array(
		0 => '',
		1 => __( 'Tag added.', 'adveits-portfolio' ),
		2 => __( 'Tag deleted.', 'adveits-portfolio' ),
		3 => __( 'Tag updated.', 'adveits-portfolio' ),
		4 => __( 'Tag not added.', 'adveits-portfolio' ),
		5 => __( 'Tag not updated.', 'adveits-portfolio' ),
		6 => __( 'Tags deleted.', 'adveits-portfolio' ),
	);

	return $messages;
}
