<?php
/**
 * Plugin Name: Adveits Portfolio
 * Description: This plugin allows you to manage, edit, and create new portfolio items in an unlimited number of portfolios.
 * Plugin URI:  https://store.adveits.com/
 * Author:      Adveits
 * Author URI:  https://www.adveits.com/
 * Version:     1.1.0
 * License:     GPL 2.0
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: adveits-portfolio
 * Domain Path: /languages   
 *
 * @package   AdveitsPortfolio
 * @version   1.1.0
 * @author    Adveits <support@adveits.com>
 * @copyright Copyright (c) 2019, Adveits
 * @link      https://store.adveits.com/
 * @license   https://www.gnu.org/licenses/gpl-2.0.html
 */

/**
 * Singleton class that sets up and initializes the plugin.
 *
 * @since  1.0.0
 * @access public
 * @return void
 */
final class ADVEITS_PORTFOLIO_Plugin {
	/**
	 * Directory path to the plugin folder.
	 *
	 * @since  1.0.0
	 * @access public
	 * @var    string
	 */
	public $dir_path = '';

	/**
	 * Directory URI to the plugin folder.
	 *
	 * @since  1.0.0
	 * @access public
	 * @var    string
	 */
	public $dir_uri = '';

	/**
	 * JavaScript directory URI.
	 *
	 * @since  1.0.0
	 * @access public
	 * @var    string
	 */
	public $js_uri = '';

	/**
	 * CSS directory URI.
	 *
	 * @since  1.0.0
	 * @access public
	 * @var    string
	 */
	public $css_uri = '';

    /**
     * IMG directory URI.
     *
     * @since  1.0.0
     * @access public
     * @var    string
     */
    public $img_uri = '';

	/**
	 * Returns the instance.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return object
	 */
	public static function get_instance() {
		static $instance = null;

		if ( is_null( $instance ) ) {
			$instance = new self;
			$instance->setup();
			$instance->includes();
			$instance->setup_actions();
		}

		return $instance;
	}

	/**
	 * Constructor method.
	 *
	 * @since  1.0.0
	 * @access private
	 * @return void
	 */
	public function __construct() {}

	/**
	 * Magic method to output a string if trying to use the object as a string.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function __toString() {
		return 'adveits-portfolio';
	}

	/**
	 * Magic method to keep the object from being cloned.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function __clone() {
		_doing_it_wrong( __FUNCTION__, __( 'Whoah, partner!', 'adveits-portfolio' ), '1.0.0' );
	}

	/**
	 * Magic method to keep the object from being unserialized.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function __wakeup() {
		_doing_it_wrong( __FUNCTION__, __( 'Whoah, partner!', 'adveits-portfolio' ), '1.0.0' );
	}

	/**
	 * Magic method to prevent a fatal error when calling a method that doesn't exist.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function __call( $method = '', $args = array() ) {
		_doing_it_wrong( "Adveits_Portfolio::{$method}", __( 'Method does not exist.', 'adveits-portfolio' ), '1.0.0' );
		unset( $method, $args );
		return null;
	}

	/**
	 * Initial plugin setup.
	 *
	 * @since  1.0.0
	 * @access private
	 * @return void
	 */
	private function setup() {
		$this->dir_path = trailingslashit( plugin_dir_path( __FILE__ ) );
		$this->dir_uri  = trailingslashit( plugin_dir_url(  __FILE__ ) );

		$this->js_uri  = trailingslashit( $this->dir_uri . 'assets/js'  );
		$this->css_uri = trailingslashit( $this->dir_uri . 'assets/css' );
        $this->img_uri = trailingslashit( $this->dir_uri . 'assets/img' );
	}

	/**
	 * Loads include and admin files for the plugin.
	 *
	 * @since  1.0.0
	 * @access private
	 * @return void
	 */
	private function includes() {
		// Load functions files
		require_once( $this->dir_path . 'inc/functions-capabilities.php' );
		require_once( $this->dir_path . 'inc/functions-filters.php' );
		require_once( $this->dir_path . 'inc/functions-options.php' );
        require_once( $this->dir_path . 'inc/functions-acf.php' );
		require_once( $this->dir_path . 'inc/functions-rewrite.php' );
		require_once( $this->dir_path . 'inc/functions-post-types.php' );
		require_once( $this->dir_path . 'inc/functions-taxonomies.php' );

		// Load template files
		require_once( $this->dir_path . 'inc/template-item.php' );
		require_once( $this->dir_path . 'inc/template-general.php' );

		// Load admin files
		if ( is_admin() ) {
			require_once( $this->dir_path . 'admin/functions-admin.php' );
			require_once( $this->dir_path . 'admin/class-manage-items.php' );
			require_once( $this->dir_path . 'admin/class-item-edit.php' );
			require_once( $this->dir_path . 'admin/class-settings.php' );
		}
	}

	/**
	 * Sets up initial actions.
	 *
	 * @since  1.1.0
	 * @access private
	 * @return void
	 */
	private function setup_actions() {
		// Internationalize the text strings used.
		add_action( 'plugins_loaded', array( $this, 'i18n' ), 2 );

		// Register activation hook.
		register_activation_hook( __FILE__, array( $this, 'activation' ) );
        register_deactivation_hook( __FILE__, 'flush_rewrite_rules' );
	}

	/**
	 * Loads the translation files.
	 *
	 * @since  1.0.0
	 * @access public
	 * @return void
	 */
	public function i18n() {
		load_plugin_textdomain( 'adveits-portfolio', false, trailingslashit( dirname( plugin_basename( __FILE__ ) ) ) . 'languages' );
	}

	/**
	 * Method that runs only when the plugin is activated.
	 *
	 * @since  1.0.0
	 * @access public
	 * @global $wpdb
	 * @return void
	 */
	public function activation() {
	    // Get the administrator role.
		$role = get_role( 'administrator' );

		// If the administrator role exists, add required capabilities for the plugin.
		if ( ! is_null( $role ) ) {
			// Taxonomy caps.
			$role->add_cap( 'manage_adveits_portfolio_categories' );
			$role->add_cap( 'edit_adveits_portfolio_categories' );
			$role->add_cap( 'delete_adveits_portfolio_categories' );
			$role->add_cap( 'assign_adveits_portfolio_categories' );

			$role->add_cap( 'manage_adveits_portfolio_tags' );
			$role->add_cap( 'edit_adveits_portfolio_tags' );
			$role->add_cap( 'delete_adveits_portfolio_tags' );
			$role->add_cap( 'assign_adveits_portfolio_tags' );

			// Post type caps.
			$role->add_cap( 'create_adveits_portfolio_items' );
			$role->add_cap( 'edit_adveits_portfolio_items' );
			$role->add_cap( 'edit_others_adveits_portfolio_items' );
			$role->add_cap( 'publish_adveits_portfolio_items' );
			$role->add_cap( 'read_private_adveits_portfolio_items' );
			$role->add_cap( 'delete_adveits_portfolio_items' );
			$role->add_cap( 'delete_private_adveits_portfolio_items' );
			$role->add_cap( 'delete_published_adveits_portfolio_items' );
			$role->add_cap( 'delete_others_adveits_portfolio_items' );
			$role->add_cap( 'edit_private_adveits_portfolio_items' );
			$role->add_cap( 'edit_published_adveits_portfolio_items' );
		}
	}
}

/**
 * Gets the instance of the `ADVEITS_PORTFOLIO_Plugin` class.  This function is useful for quickly grabbing data
 * used throughout the plugin.
 *
 * @since  1.0.0
 * @access public
 * @return object
 */
function adveits_portfolio_plugin() {
	return ADVEITS_PORTFOLIO_Plugin::get_instance();
}

adveits_portfolio_plugin();
